import { describe, it, expect, vi, beforeEach, afterEach, Mock } from "vitest";
import { Server } from "./Server";

vi.mock("@tsed/common", async (importOriginal) => {
  const actual: any = await importOriginal();
  return {
    ...actual,
    $log: {
      error: vi.fn()
    }
  };
});

vi.mock("@tsed/platform-express", async (importOriginal) => {
  const actual: any = await importOriginal();
  return {
    ...actual,
    PlatformExpress: {
      bootstrap: vi.fn()
    }
  };
});

describe("bootstrap function", () => {
  let bootstrap: () => Promise<void>;
  let mockPlatform: {
    listen: Mock;
    stop: Mock;
  };

  beforeEach(async () => {
    vi.clearAllMocks();

    mockPlatform = {
      listen: vi.fn().mockResolvedValue(undefined),
      stop: vi.fn().mockResolvedValue(undefined)
    };

    const { PlatformExpress } = await import("@tsed/platform-express");
    (PlatformExpress.bootstrap as Mock).mockResolvedValue(mockPlatform);

    const module = await import("./index");
    bootstrap = module.bootstrap;
  });

  afterEach(() => {
    vi.restoreAllMocks();
  });

  it("should bootstrap the server successfully", async () => {
    await bootstrap();
    const { PlatformExpress } = await import("@tsed/platform-express");
    expect(PlatformExpress.bootstrap).toHaveBeenCalledWith(Server);
    expect(mockPlatform.listen).toHaveBeenCalled();
  });

  it("should handle errors during bootstrap", async () => {
    const error = new Error("Bootstrap error");
    const { PlatformExpress } = await import("@tsed/platform-express");
    (PlatformExpress.bootstrap as Mock).mockRejectedValueOnce(error);

    await bootstrap();

    const { $log } = await import("@tsed/common");
    expect($log.error).toHaveBeenCalledWith({
      event: "SERVER_BOOTSTRAP_ERROR",
      message: error.message,
      stack: error.stack
    });
  });

  it("should set up SIGINT handler", async () => {
    const processOnSpy = vi.spyOn(process, "on");
    await bootstrap();
    expect(processOnSpy).toHaveBeenCalledWith("SIGINT", expect.any(Function));

    // Simulate SIGINT
    const sigintHandler = processOnSpy.mock.calls.find((call) => call[0] === "SIGINT")?.[1];
    if (sigintHandler && typeof sigintHandler === "function") {
      sigintHandler();
      expect(mockPlatform.stop).toHaveBeenCalled();
    } else {
      throw new Error("SIGINT handler not found or not a function");
    }
  });
});
