import { describe, it, expect, vi, beforeEach, afterAll, beforeAll } from "vitest";
import { PlatformTest, Req } from "@tsed/common";
import { UserService } from "../services/UserService";
import { Server } from "../Server";
import { sqliteDatasource } from "../datasources/SqliteDatasource";
import { GithubProtocol } from "./GthubProtocol";

describe("GithubProtocol", () => {
  let protocol: GithubProtocol;
  let userService: UserService;

  beforeAll(async () => {
    await PlatformTest.create({ platform: Server, imports: [sqliteDatasource] }); // ensure PlatformTest.create() is called
  });

  afterAll(() => {
    return PlatformTest.reset();
  });

  beforeEach(async () => {
    userService = {
      findOrCreate: vi.fn().mockResolvedValue({ id: "user123", username: "githubuser" })
    } as unknown as UserService;

    protocol = await PlatformTest.invoke<GithubProtocol>(GithubProtocol, [{ token: UserService, use: userService }]);
  });

  it("should call $onVerify and return a user", async () => {
    const mockReq = {
      query: { state: "github-state" }
    } as unknown as Req;
    const mockAccessToken = "mock-access-token";
    const mockProfile = { username: "githubuser" };

    const fetchSpy = vi.spyOn(protocol, "fetchVerifiedEmails").mockResolvedValue([{ value: "user@example.com", verified: true }]);

    const result = await protocol.$onVerify(mockReq, mockAccessToken, "", mockProfile);

    expect(fetchSpy).toHaveBeenCalledWith(mockAccessToken);
    expect(userService.findOrCreate).toHaveBeenCalledWith({
      service: "github",
      serviceIdentifier: "github-state",
      username: "githubuser",
      emails: [{ value: "user@example.com", verified: true }],
      accessToken: mockAccessToken
    });
    expect(result).toEqual({ id: "user123", username: "githubuser" });
  });

  it("should throw an error if no verified emails are found", async () => {
    const mockReq = { query: { state: "github-state" } } as unknown as Req;
    const mockAccessToken = "mock-access-token";
    const mockProfile = { username: "githubuser" };

    vi.spyOn(protocol, "fetchVerifiedEmails").mockResolvedValue([]);

    await expect(protocol.$onVerify(mockReq, mockAccessToken, "", mockProfile)).rejects.toThrow("No verified email found");
  });

  it("should fetch verified emails from GitHub", async () => {
    global.fetch = vi.fn().mockResolvedValue({
      json: vi.fn().mockResolvedValue([
        { value: "email1@example.com", verified: true },
        { value: "email2@example.com", verified: false }
      ])
    });

    const emails = await protocol.fetchVerifiedEmails("mock-access-token");

    expect(emails).toEqual([{ value: "email1@example.com", verified: true }]);
    expect(global.fetch).toHaveBeenCalledWith("https://api.github.com/user/emails", expect.anything());
  });
});
