// protocols/GithubProtocol.ts
import { Protocol, OnVerify, OnInstall } from "@tsed/passport";
import { Req } from "@tsed/common";
import { Inject } from "@tsed/di";
import { UserService } from "../services/UserService";
import { Strategy as GithubStrategy } from "passport-github";
import { SqliteDatasource } from "../datasources/SqliteDatasource";

@Protocol({
  name: "github",
  useStrategy: GithubStrategy,
  settings: {
    clientID: process.env.GITHUB_CLIENT_ID || "your-client-id",
    clientSecret: process.env.GITHUB_CLIENT_SECRET || "your-client-secret",
    callbackURL: "http://localhost:8080/auth/github/callback",
    scope: ["user:email"],
    state: true,
    passReqToCallback: true
  }
})
export class GithubProtocol implements OnVerify, OnInstall {
  @Inject()
  userService: UserService;

  @Inject()
  sqliteDatasource: SqliteDatasource;

  async $onVerify(@Req() req: Req, accessToken: string, _refreshToken: string, profile: any) {
    const emails = await this.fetchVerifiedEmails(accessToken);

    if (!emails.length) {
      throw new Error("No verified email found");
    }

    const state = req.query.state;
    let identifier: string;

    if (typeof state === "string") {
      identifier = state;
    } else if (Array.isArray(state)) {
      // If state is an array, take the first string element
      identifier = state.find((s) => typeof s === "string") as string;
      if (!identifier) {
        throw new Error("Invalid service identifier");
      }
    } else {
      throw new Error("Service identifier is missing or invalid");
    }

    const user = await this.userService.findOrCreate({
      service: "github",
      serviceIdentifier: identifier.toString(),
      username: profile.username,
      emails: emails,
      accessToken: accessToken
    });

    return user;
  }

  async fetchVerifiedEmails(accessToken: string): Promise<any[]> {
    const response = await fetch("https://api.github.com/user/emails", {
      headers: {
        Authorization: `token ${accessToken}`,
        "User-Agent": "YourAppName"
      }
    });

    const emails = await response.json();
    return emails.filter((email: any) => email.verified);
  }

  $onInstall(strategy: GithubStrategy) {
    console.log("Github strategy installed");
    if (process.env.NODE_ENV === "development") {
      console.log(strategy);
    }
    // Optional: additional strategy configuration
  }
}
